@@ -20,13 +20,14 @@ class Channel(virtual.Channel):
         Queue.objects.publish(queue, serialize(message))
 
     def basic_consume(self, queue, *args, **kwargs):
-        exchange, _ = self.state.bindings[queue]
+        qinfo = self.state.bindings[queue]
+        exchange = qinfo[0]
         if self.typeof(exchange).type == "fanout":
             return
         super(Channel, self).basic_consume(queue, *args, **kwargs)
 
     def _get(self, queue):
-        self.refresh_connection()
+        #self.refresh_connection()
         m = Queue.objects.fetch(queue)
         if m:
             return deserialize(m)
